args <- commandArgs( trailingOnly=TRUE )

samplesFile  <- args[1]
expsamplesFile <- args[2]
titlestr     <- args[3]

samples  <- as.matrix(read.table( samplesFile, sep=',', header=FALSE ))
expsamples <- as.matrix(read.table( expsamplesFile, sep=',', header=FALSE))

# Compute isotonic regression for estimated actual spectrum
e <- isoreg( samples[,1], -1*samples[,2] )
e$yf <- -1*e$yf

# Compute isotonic regression for estimated expected spectrum
efake <- isoreg( expsamples[,1], -1*expsamples[,2] )
efake$yf <- -1*efake$yf

# Start the plotting magic
quartz( width=5, height=5 )
par( mai=c(0.65, 0.65, 0.35, 0.2) )
plot( samples, log='y', type='p', col='gray',
     cex=0.5, main=titlestr, xlab=NA, ylab=NA )
lines(e, col='blue', lwd=2)
lines(efake, col='green', lwd=2)
mtext(side = 1, expression(sigma), line = 2)
mtext(side = 2, "count", line = 2, font = 2)
quartz.save( sprintf( "spectra-%s.pdf", titlestr ), type="pdf" )
